package com.zcxk.charge.core.dao;

import com.zcxk.charge.api.dto.MonthRevenueDto;
import com.zcxk.charge.api.dto.MonthSellwaterDto;
import com.zcxk.charge.api.vo.MonthRevenueVo;
import com.zcxk.charge.api.vo.MonthSellwaterVo;
import com.zcxk.charge.core.entity.MonthRevenueEntity;
import com.zcxk.charge.core.entity.MonthSellwaterEntity;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

/**
 * @description
 * @author yjy
 * @data 2021-2-24
 */
@Mapper
public interface RevenueMapper {

    /**
     * 查询本月售水信息
     * @param monthSellwaterDto 售水信息
     * @return
     */
    List<MonthSellwaterVo> selectMonthSell(MonthSellwaterDto monthSellwaterDto);

    /**
     * 查询近6个月的售水量
     * @param monthSellwaterDto 售水信息
     * @return
     */
    List<MonthSellwaterVo> selectMonthSellTotal(MonthSellwaterDto monthSellwaterDto);

    /**
     * 查询近几个月的月营收情况
     * @param monthRevenueDto 营收信息
     * @return
     */
    List<MonthRevenueVo> selectMonthRevenue(MonthRevenueDto monthRevenueDto);

    /**
     * 查询本月各分公司的营收情况
     * @param monthRevenueDto 营收信息
     * @return
     */
    List<MonthRevenueVo> selectCompanyRevenue(MonthRevenueDto monthRevenueDto);

    /**
     * 查询App总览售水数据
     * @param monthRevenueDto 营收信息
     * @return
     */
    List<MonthRevenueVo> selectOverviewRevenue(MonthRevenueDto monthRevenueDto);

    /**
     * 查询App总览售水环比
     * @param monthRevenueDto 营收信息
     * @return
     */
    List<MonthRevenueVo> selectRevenueByDate(MonthRevenueDto monthRevenueDto);

    /**
     * 查询是否当月有营收数据
     * @param tenantId 租户id
     * @return
     */
    Integer isExistThisMonthRevenue(String tenantId);

    /**
     * 插入营收数据
     * @param monthRevenueEntity 营收数据
     * @return
     */
    Integer insert(MonthRevenueEntity monthRevenueEntity);

    /**
     * 修改营收数据
     * @param monthRevenueEntity 营收数据
     * @return
     */
    Integer update(MonthRevenueEntity monthRevenueEntity);

    /**
     * 查询该租户一级公司id
     * @param tenantId 租户id
     * @return
     */
    Integer selectCompanyIdByTenant(String tenantId);

    /**
     * 插入售水数据
     * @param monthSellwaterEntity 售水信息
     * @return
     */
    Integer insertSellWater(MonthSellwaterEntity monthSellwaterEntity);
    /**
     * 修改售水数据
     * @param monthSellwaterEntity 售水信息
     * @return
     */
    Integer updateSellWater(MonthSellwaterEntity monthSellwaterEntity);
    /**
     * 修改营业数据
     * @param monthRevenueEntity 营业信息
     * @return
     */
    Integer updateRevenue(MonthRevenueEntity monthRevenueEntity);
    /**
     * 查询月营收情况
     * @param monthRevenueDto 营业信息查询条件
     * @return
     */
    List<MonthRevenueVo> selectList(MonthRevenueDto monthRevenueDto);
    /**
     * 查询某月是否有营收数据
     * @param companyOrgId 机构id
     * @param collectDateString 时间
     * @param id 主键
     * @return
     */
    Integer isExistMonthRevenue(Integer companyOrgId, String collectDateString, Integer id);
}
